@@ -1,4 +1,4 @@
-__author__ = 'Carlos Alberto Gomez Gonzalez'
+__author__ = 'Carlos Alberto Gomez Gonzalez, Valentin Christiaens'
 __all__ = ['plot_frames',
            'plot_cubes']
 
@@ -16,7 +16,7 @@ from mpl_toolkits.axes_grid1 import make_axes_locatable
 from matplotlib.cm import register_cmap
 import matplotlib.colors as colors
 from matplotlib.colors import LinearSegmentedColormap
-import matplotlib.cm as mplcm
+#import matplotlib.cm as mplcm
 import warnings
 warnings.filterwarnings("ignore", module="matplotlib")
 
@@ -37,16 +37,17 @@ default_cmap = 'viridis'
 def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
                 vmin=None, circle=None, circle_alpha=0.8, circle_color='white',
                 circle_linestyle='-', circle_radius=6, circle_label=False, 
-                arrow=None, arrow_alpha=0.8, arrow_length=10, arrow_shiftx=5, 
-                arrow_label=None, label=None, label_pad=5, label_size=12, 
-                label_color='white',grid=False, grid_alpha=0.4,  grid_color='#f7f7f7', 
-                grid_spacing=None, cross=None, cross_alpha=0.4, lab_fontsize=8,
-                cross_color='white', ang_scale=False, ang_ticksep=50, ndec=1, 
-                pxscale=0.01, auscale=1., ang_legend=False, au_legend=False, 
-                axis=True, show_center=False, cmap=None, log=False, 
-                colorbar=True, colorbar_ticks=None, dpi=100, size_factor=6, 
-                horsp=0.4, versp=0.2, width=400, height=400, title=None, 
-                tit_size=16, sampling=1, save=None, transparent=False):
+                circle_label_color='white', arrow=None, arrow_alpha=0.8, 
+                arrow_length=10, arrow_shiftx=5, arrow_label=None, label=None, 
+                label_pad=5, label_size=12, label_color='white',grid=False,
+                grid_alpha=0.4,  grid_color='#f7f7f7', grid_spacing=None, 
+                cross=None, cross_alpha=0.4, lab_fontsize=8, cross_color='white', 
+                ang_scale=False, ang_ticksep=50, ndec=1, pxscale=0.01, 
+                auscale=1., ang_legend=False, au_legend=False, axis=True, 
+                show_center=False, cmap=None, log=False, colorbar=True, 
+                colorbar_ticks=None, dpi=100, size_factor=6, horsp=0.4, 
+                versp=0.2, width=400, height=400, title=None, tit_size=16, 
+                sampling=1, save=None, transparent=False):
     """ Plot a 2d array or a tuple of 2d arrays. Supports the ``matplotlib`` and
     ``bokeh`` backends. When having a tuple of 2d arrays, the plot turns into a
     mosaic. For ``matplotlib``, instead of a mosaic of images, we can create a
@@ -88,6 +89,9 @@ def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
         [backend='matplotlib'] Whether to show the coordinates next to each
         circle. If a string: the string to be printed. If a tuple, should be 
         a tuple of strings with same length as 'circle'.
+    circle_label_color : str, optional
+        [backend='matplotlib'] Default 'white'. Sets the color of the circle
+        label
     arrow : None or tuple of floats, optional
         [backend='matplotlib'] To show an arrow pointing to the given pixel
         coordinates.
@@ -355,7 +359,7 @@ def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
     show_center = check_bool_param(show_center, 'show_center')
     ang_scale = check_bool_param(ang_scale, 'ang_scale')
     ang_legend = check_bool_param(ang_legend, 'ang_legend')
-    au_legend = check_bool_param(au_legend, 'ang_legend')
+    au_legend = check_bool_param(au_legend, 'au_legend')
 
     if isinstance(grid_color, str):
         grid_color = [grid_color] * num_plots
@@ -423,15 +427,15 @@ def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
 
             if plot_mosaic:
                 ax = subplot(rows, cols, v)
-                ax.set_aspect('equal')
+                ax.set_aspect('auto')
 
                 if logscale[i]:
-                    image += np.abs(image.min())
+                    image += np.abs(np.nanmin(image))
                     if vmin[i] is None:
                         linthresh = 1e-2
                     else:
                         linthresh = vmin[i]
-                    norm = colors.SymLogNorm(linthresh)
+                    norm = colors.SymLogNorm(linthresh, base=10)
                 else:
                     norm = None
 
@@ -450,16 +454,18 @@ def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
                 im = ax.imshow(image, cmap=cucmap, origin='lower', norm=norm,
                                interpolation='nearest', vmin=vmin[i],
                                vmax=vmax[i])
-
-                if colorbar[i]:
-                    divider = make_axes_locatable(ax)
-                    # the width of cax is 5% of ax and the padding between cax
-                    # and ax wis fixed at 0.05 inch
-                    cax = divider.append_axes("right", size="5%", pad=0.05)
-                    cb = plt_colorbar(im, ax=ax, cax=cax, drawedges=False,
-                                      ticks=cbticks)
-                    cb.outline.set_linewidth(0.1)
-                    cb.ax.tick_params(labelsize=8)
+                # if colorbar[i]:
+                #     divider = make_axes_locatable(ax)
+                #     # the width of cax is 5% of ax and the padding between cax
+                #     # and ax wis fixed at 0.05 inch
+                #     cax = divider.append_axes("right", size="5%", pad=0.05)
+                #     cb = plt_colorbar(im, ax=ax, cax=cax, drawedges=False,
+                #                       ticks=cbticks)
+                #     cb.outline.set_linewidth(0.1)
+                #     cb.ax.tick_params(labelsize=lab_fontsize)
+                #     cbw = frame_size/10
+                # else:
+                #     cbw = 0
 
             else:
                 # Leave the import to make porjection='3d' work
@@ -467,7 +473,7 @@ def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
                 x = np.outer(np.arange(0, frame_size, 1), np.ones(frame_size))
                 y = x.copy().T
                 ax = subplot(rows, cols, v, projection='3d')
-                ax.set_aspect('equal')
+                ax.set_aspect('auto')
                 surf = ax.plot_surface(x, y, image, rstride=sampling,
                                        cstride=sampling, linewidth=2,
                                        cmap=custom_cmap[i], antialiased=True,
@@ -496,7 +502,7 @@ def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
                        linestyles='solid')
                 annotate(scalab, (xmi + scalabloc, scapad + 2), color='white', 
                          size=label_size)
-            elif au_legend[i] and plot_mosaic:
+            if au_legend[i] and plot_mosaic:
                 pxsc_fac = (0.012265/pxscale)
                 labsz_fac = (label_size/12)
                 scaleng = 50. / auscale
@@ -513,8 +519,8 @@ def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
                 scapad = 5*pxsc_fac*labsz_fac
                 xma = frame_size - scapad
                 xmi = xma - scaleng
-                hlines(y=xma-scapad, xmin=xmi, xmax=xma, colors='white', lw=1.,
-                       linestyles='solid')
+                hlines(y=xma-scapad, xmin=xmi, xmax=xma, colors='white', 
+                       lw=1., linestyles='solid')
                 annotate(scalab, (xmi + scalabloc, xma-0.5*scapad), 
                          color='white', size=label_size)
                 
@@ -523,11 +529,19 @@ def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
                     c_offset = circle_linestyle[0]
                     circle_linestyle = circle_linestyle[1]
                 else:
-                    c_offset = 2
+                    c_offset = lab_fontsize+1  # vertical offset is equal to the font size + 1, was 2
                 for j in range(n_circ):
+                    if isinstance(circle_color, (list, tuple)):
+                        circle_color_tmp = circle_color[j]
+                    else:
+                        circle_color_tmp = circle_color
+                    if isinstance(circle_linestyle, (list, tuple)):
+                        circle_linestyle_tmp = circle_linestyle[j]
+                    else:
+                        circle_linestyle_tmp = circle_linestyle
                     circ = Circle(coor_circle[j], radius=circle_radius[j],
-                                  fill=False, color=circle_color,
-                                  alpha=circle_alpha[j], ls=circle_linestyle)
+                                  fill=False, color=circle_color_tmp,
+                                  alpha=circle_alpha[j], ls=circle_linestyle_tmp)
                     ax.add_artist(circ)
                     if circle_label:                  
                         x = coor_circle[j][0]
@@ -539,7 +553,7 @@ def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
                         else:
                             cirlabel = str(int(x))+','+str(int(y))
                         ax.text(x, y + circle_radius[j] + c_offset, cirlabel,
-                                fontsize=lab_fontsize, color='white', family='monospace',
+                                fontsize=lab_fontsize, color=circle_label_color, family='monospace',
                                 ha='center', va='center', weight='bold',
                                 alpha=circle_alpha[j])
 
@@ -548,8 +562,6 @@ def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
                            linestyle='dashed', color='white')
                 ax.axvline(coor_cross[1], ymin=0, ymax=frame_size, alpha=cross_alpha, lw=0.6,
                            linestyle='dashed', color='white')
-#                ax.scatter([coor_cross[0]], [coor_cross[1]], marker='+',
-#                           color=cross_color, alpha=cross_alpha)
 
             if show_center[i] and plot_mosaic:
                 ax.scatter([cy], [cx], marker='+',
@@ -625,8 +637,8 @@ def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
                     labels_x.append(round(Decimal(t * (ang_ticksep * pxscale)),ndec))
                 ax.set_xticklabels(labels_x)
                 ax.set_yticklabels(labels_y)
-                ax.set_xlabel("arcsec", fontsize=label_size)
-                ax.set_ylabel("arcsec", fontsize=label_size)
+                ax.set_xlabel('\u0394RA["]', fontsize=label_size)
+                ax.set_ylabel('\u0394Dec["]', fontsize=label_size)
                 ax.tick_params(axis='both', which='major', labelsize=label_size)
             else:
                 ax.set_xlabel("x", fontsize=label_size)
@@ -634,6 +646,16 @@ def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
 
             if not axis[i]:
                 ax.set_axis_off()
+                
+            if colorbar[i]:
+                divider = make_axes_locatable(ax)
+                # the width of cax is 5% of ax and the padding between cax
+                # and ax wis fixed at 0.05 inch
+                cax = divider.append_axes("right", size="5%", pad=0.05)
+                cb = plt_colorbar(im, ax=ax, cax=cax, drawedges=False,
+                                  ticks=cbticks)
+                cb.outline.set_linewidth(0.1)
+                cb.ax.tick_params(labelsize=lab_fontsize)
 
         fig.subplots_adjust(wspace=horsp, hspace=versp)
         if save is not None and isinstance(save, str):
@@ -646,20 +668,21 @@ def plot_frames(data, backend='matplotlib', mode='mosaic', rows=1, vmax=None,
     elif backend == 'bokeh':
         hv.extension(backend)
         subplots = []
-        options = "Image (cmap='" + custom_cmap[0] + "')"  # taking first item
-        hv.opts(options)
+        # options = "Image (cmap='" + custom_cmap[0] + "')"  # taking first item
+        # hv.opts(options)
 
         for i, v in enumerate(range(num_plots)):
             image = data[i].copy()
             if vmin[i] is None:
-                vmin_i = image.min()
+                vmin[i] = image.min()
             if vmax[i] is None:
-                vmax_i = image.max()
+                vmax[i] = image.max()
             im = hv.Image((range(image.shape[1]), range(image.shape[0]), image))
             subplots.append(im.opts(tools=['hover'], colorbar=colorbar[i],
                                     colorbar_opts={'width': 15},
                                     width=width, height=height,
-                                    clim=(vmin_i, vmax_i)))
+                                    clim=(vmin[i], vmax[i]),
+                                    cmap=custom_cmap[0]))
 
         return hv.Layout(subplots).cols(cols)
 
@@ -798,15 +821,17 @@ def plot_cubes(cube, mode='slider', backend='matplotlib', dpi=100,
             # keywords in the currently active 'matplotlib' renderer are:
             # 'alpha', 'clims', 'cmap', 'filterrad', 'interpolation', 'norm',
             # 'visible'
-            options = "Image (cmap='" + cmap + "', interpolation='nearest',"
-            options += " clims=("+str(vmin)+','+str(vmax)+")"+")"
-            opts(options, image_stack)
-            return image_stack.opts(opts.Image(colorbar=colorbar))
+            #options = "Image (cmap='" + cmap + "', interpolation='nearest',"
+            #options += " clims=("+str(vmin)+','+str(vmax)+")"+")"
+            #opts(options, image_stack)
+            return image_stack.opts(opts.Image(colorbar=colorbar,
+                                               cmap=cmap,
+                                               clim=(vmin, vmax)))
             # hv.save(image_stack, 'holomap.gif', fps=5)
 
         elif backend == 'bokeh':
-            options = "Image (cmap='" + cmap + "')"
-            opts(options, image_stack)
+            #options = "Image (cmap='" + cmap + "')"
+            #opts(options, image_stack)
             # Compensating the width to accommodate the colorbar
             if colorbar:
                 cb_wid = 15
@@ -827,6 +852,7 @@ def plot_cubes(cube, mode='slider', backend='matplotlib', dpi=100,
                                                               'padding': 3},
                                                width=width_, height=height,
                                                clim=(vmin, vmax),
+                                               cmap=cmap,
                                                tools=['hover']))
 
     elif mode == 'animation':
